# -*- coding: utf-8 -*-
from zExceptions import Forbidden
from ZPublisher.HTTPRequest import HTTPRequest


def check(request):
    if isinstance(request, HTTPRequest):
        if request.get('REQUEST_METHOD', 'GET').upper() != 'POST':
            raise Forbidden('Request must be POST')



# -*- coding: utf-8 -*-
from plone.protect.postonly import check
from unittest import makeSuite
from unittest import TestCase
from unittest import TestSuite
from zExceptions import Forbidden
from ZPublisher.HTTPRequest import HTTPRequest


class PostOnlyTests(TestCase):

    def makeRequest(self, method):
        return HTTPRequest(None,
                           dict(REQUEST_METHOD=method,
                                SERVER_PORT="80",
                                SERVER_NAME="localhost"),
                           None)

    def testNonHTTPRequestAllowed(self):
        check("not a request")

    def testGETRequestForbidden(self):
        self.assertRaises(Forbidden, check, self.makeRequest("GET"))

    def testPOSTRequestAllowed(self):
        check(self.makeRequest("POST"))


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(PostOnlyTests))
    return suite



# -*- coding: utf-8 -*-
from plone.protect.testing import PROTECT_FUNCTIONAL_TESTING
from zExceptions import Forbidden
from zope.component import getMultiAdapter

import unittest


class TestAttackVector(unittest.TestCase):
    layer = PROTECT_FUNCTIONAL_TESTING

    def test_regression(self):
        portal = self.layer['portal']
        request = self.layer['request']
        view = getMultiAdapter(
            (portal, request), name=u'confirm-action')
        request.form.update({
            'original_url': 'foobar'
        })
        self.assertTrue('value="foobar"' in view())

    def test_valid_url(self):
        portal = self.layer['portal']
        request = self.layer['request']
        view = getMultiAdapter(
            (portal, request), name=u'confirm-action')
        request.form.update({
            'original_url': 'javascript:alert(1)'
        })
        self.assertRaises(Forbidden, view)



# -*- coding: utf-8 -*-
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.protect.testing import PROTECT_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser

import unittest


class TestCSRF(unittest.TestCase):
    layer = PROTECT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.browser = Browser(self.layer['app'])
        self.request = self.layer['request']
        self.browser.addHeader(
            'Authorization', 'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,))

    def test_change_password_on_root_does_not_throw_other_csrf_protection(
            self):
        self.browser.open('%s/acl_users/users/manage_users?user_id=%s&passwd=1' % (
            self.layer['app'].absolute_url(), SITE_OWNER_NAME))
        self.browser.getControl(name='password').value = SITE_OWNER_PASSWORD
        self.browser.getControl(name='confirm').value = SITE_OWNER_PASSWORD
        self.browser.getForm().submit()
        self.assertEquals(
            self.browser.url,
            '%s/acl_users/users/manage_users?manage_tabs_message=password+updated' % (
                self.layer['app'].absolute_url()))



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing.layers import FunctionalTesting
from plone.protect.auto import safeWrite
from Products.Five import BrowserView
from zope.configuration import xmlconfig


class ProtectedLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # load ZCML
        import plone.protect
        xmlconfig.file('configure.zcml', plone.protect,
                       context=configurationContext)
        xmlconfig.file('test.zcml', plone.protect.tests,
                       context=configurationContext)

    def setUpPloneSite(self, portal):
        # install into the Plone site
        applyProfile(portal, 'plone.protect:default')
        self.portal = portal


PROTECT_FIXTURE = ProtectedLayer()
PROTECT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PROTECT_FIXTURE,), name="PROTECT_FIXTURE:Functional")


class TestUnprotectedView(BrowserView):

    def __call__(self):
        # on posts, write something to the db
        if 'submit1' in self.request.form or 'submit2' in self.request.form:
            self.context.foo = 'bar'
            self.context._p_changed = True
        return """
<html>
<body>
<form id="one" method="POST">
    <input type="submit" name="submit1" value="submit1" />
</form>
<form id="two" action="%s" METHOD="post">
    <input type="submit" name="submit2" value="submit2" />
</form>
<form id="three" method="GET">
    <input type="submit" name="submit3" value="submit3" />
</form>
<form id="four" action="//foobar/somepath.php" method="POST">
    <input type="submit" name="submit4" value="submit4" />
</form>
<form id="five" action="//nohost/plone" method="POST">
    <input type="submit" name="submit5" value="submit5" />
</form>
<form id="six" action="https://foobar/somepath.php" method="POST">
    <input type="submit" name="submit6" value="submit6" />
</form>
<form id="seven" action="a/relative/path" method="POST">
    <input type="submit" name="submit7" value="submit7" />
</form>
</body>
</html>""" % (
            self.request.URL,
        )


class TestSafeToWriteObject(BrowserView):

    def __call__(self):
        self.context.foo = 'bar'
        safeWrite(self)
        return 'done'



# -*- coding: utf-8 -*-
from AccessControl.requestmethod import _buildFacade
from Acquisition import aq_parent
from OFS.interfaces import IApplication
from plone.keyring.keymanager import KeyManager
from plone.protect.authenticator import createToken
from zope.globalrequest import getRequest

import inspect
import logging


SAFE_WRITE_KEY = 'plone.protect.safe_oids'
LOGGER = logging.getLogger('plone.protect')

_default = []

# This is based on AccessControl.requestmethod.postonly
# It should probably be updated to use the decorator module.


class protect(object):

    def __init__(self, *checkers):
        self.checkers = checkers

    def __call__(self, callable):
        spec = inspect.getargspec(callable)
        args, defaults = spec[0], spec[3]
        try:
            r_index = args.index("REQUEST")
        except ValueError:
            raise ValueError("No REQUEST parameter in callable signature")

        arglen = len(args)
        if defaults is not None:
            defaults = zip(args[arglen - len(defaults):], defaults)
            arglen -= len(defaults)

        def _curried(*args, **kw):
            request = None

            if len(args) > r_index:
                request = args[r_index]

            for checker in self.checkers:
                checker(request)

            # Reconstruct keyword arguments
            if defaults is not None:
                args, kwparams = args[:arglen], args[arglen:]
                for positional, (key, default) in zip(kwparams, defaults):
                    if positional is _default:
                        kw[key] = default
                    else:
                        kw[key] = positional

            return callable(*args, **kw)

        # Build a facade, with a reference to our locally-scoped _curried
        facade_globs = dict(_curried=_curried, _default=_default)
        try:
            name = callable.__name__
            exec _buildFacade(name, spec, callable.__doc__) in facade_globs
        except TypeError:  # BBB: Zope 2.10
            name = '_facade'
            exec _buildFacade(spec, callable.__doc__) in facade_globs
        return facade_globs[name]


def addTokenToUrl(url, req=None, manager=None):
    if not url:
        return url
    if req is None:
        req = getRequest()
    if req is None or not url.startswith(req.SERVER_URL):
        # only transforms urls to same site
        return url
    if getattr(req, 'environ', _default) is _default:
        # TestRequests have no environ.
        token = createToken(manager=manager)
    elif '_auth_token' not in req.environ:
        # Let's cache this value since this could be called
        # many times for one request.
        token = createToken(manager=manager)
        req.environ['_auth_token'] = token
    else:
        token = req.environ['_auth_token']

    if '_authenticator' not in url:
        if '?' not in url:
            url += '?'
        else:
            url += '&'
        url += '_authenticator=' + token
    return url


def getRootKeyManager(root):
    if not IApplication.providedBy(root):
        return
    try:
        manager = root._key_manager
    except AttributeError:
        manager = root._key_manager = KeyManager()
        safeWrite(root._key_manager)
        safeWrite(root)
    return manager


def getRoot(context):
    while not IApplication.providedBy(context) and context is not None:
        context = aq_parent(context)
    return context


def safeWrite(obj, request=None):
    if request is None:
        request = getRequest()
    if request is None or getattr(request, 'environ', _default) is _default:
        # Request not found or it is a TestRequest without an environment.
        LOGGER.debug('could not mark object as a safe write')
        return
    if SAFE_WRITE_KEY not in request.environ:
        request.environ[SAFE_WRITE_KEY] = []
    try:
        if obj._p_oid not in request.environ[SAFE_WRITE_KEY]:
            request.environ[SAFE_WRITE_KEY].append(obj._p_oid)
    except AttributeError:
        LOGGER.debug('object you attempted to mark safe does not have an oid')



# -*- coding: utf-8 -*-
from plone.protect.testing import PROTECT_FUNCTIONAL_TESTING
from plone.protect.utils import addTokenToUrl
from plone.protect.utils import protect
from unittest import makeSuite
from unittest import TestCase
from unittest import TestSuite

import unittest


def funcWithoutRequest():
    pass


def funcWithRequest(one, two, REQUEST=None):
    return (one, two)


class DummyChecker:

    def __call__(self, request):
        self.request = request


class DecoratorTests(TestCase):

    def testFunctionMustHaveRequest(self):
        protector = protect()
        self.assertRaises(ValueError, protector, funcWithoutRequest)

    def testArgumentsPassed(self):
        wrapped = protect()(funcWithRequest)
        self.assertEqual(wrapped("one", "two"), ("one", "two"))

    def testKeywordArguments(self):
        wrapped = protect()(funcWithRequest)
        self.assertEqual(wrapped(one="one", two="two"), ("one", "two"))

    def testMixedArguments(self):
        wrapped = protect()(funcWithRequest)
        self.assertEqual(wrapped("one", two="two"), ("one", "two"))

    def testRequestPassedToChecker(self):
        checker = DummyChecker()
        wrapped = protect(checker)(funcWithRequest)
        request = []
        wrapped("one", "two", request)
        self.failUnless(checker.request is request)


class UrlTests(unittest.TestCase):

    layer = PROTECT_FUNCTIONAL_TESTING

    def testWithUrlFromSameDomain(self):
        url = addTokenToUrl('http://nohost/foobar', self.layer['request'])
        self.failUnless('_authenticator=' in url)

    def testWithUrlFromOtherDomain(self):
        url = addTokenToUrl('http://other/foobar', self.layer['request'])
        self.failUnless('_authenticator=' not in url)

    def testAddingWithQueryParams(self):
        url = addTokenToUrl('http://nohost/foobar?foo=bar',
                            self.layer['request'])
        self.failUnless('_authenticator=' in url)

    def testWithoutRequest(self):
        url = addTokenToUrl('http://nohost/foobar')
        self.failUnless('_authenticator=' in url)

    def testWithNone(self):
        url = addTokenToUrl(None, self.layer['request'])
        self.failUnless(not url)


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(DecoratorTests))
    suite.addTest(makeSuite(UrlTests))
    return suite



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from Acquisition import aq_parent
from BTrees.OOBTree import OOBTree
from lxml import etree
from lxml import html
from plone.keyring.interfaces import IKeyManager
from plone.portlets.interfaces import IPortletAssignment
from plone.protect.authenticator import check
from plone.protect.authenticator import createToken
from plone.protect.authenticator import isAnonymousUser
from plone.protect.interfaces import IConfirmView
from plone.protect.interfaces import IDisableCSRFProtection
from plone.protect.utils import getRoot
from plone.protect.utils import getRootKeyManager
from plone.protect.utils import SAFE_WRITE_KEY
from plone.protect.utils import safeWrite  # noqa b/w compat import
from plone.transformchain.interfaces import ITransform
from Products.CMFCore.utils import getToolByName
from repoze.xmliter.serializer import XMLSerializer
from repoze.xmliter.utils import getHTMLSerializer
from six.moves.urllib.parse import urlencode
from six.moves.urllib.parse import urlparse
from zExceptions import Forbidden
from zope.component import adapts
from zope.component import ComponentLookupError
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Interface

import itertools
import logging
import os
import pkg_resources
import six
import traceback
import transaction
import types


try:
    from zope.component.hooks import getSite
except:
    from zope.app.component.hooks import getSite

try:
    pkg_resources.get_distribution('plone.app.blob')
except pkg_resources.DistributionNotFound:
    ATBlob = None
else:
    from plone.app.blob.content import ATBlob


logger = logging.getLogger('plone.protect')

X_FRAME_OPTIONS = os.environ.get('PLONE_X_FRAME_OPTIONS', 'SAMEORIGIN')
CSRF_DISABLED = os.environ.get('PLONE_CSRF_DISABLED', 'false').lower() in \
    ('true', 't', 'yes', 'y', '1')
ANNOTATION_KEYS = (
    'plone.contentrules.localassignments',
    'syndication_settings',
    'plone.portlets.contextassignments',
    # plone.scale is new compared to plone4.csrffixes:
    'plone.scale',
)


@implementer(ITransform)
class ProtectTransform(object):
    """
    XXX Need to be extremely careful with everything we do in here
    since an error here would mean the transform is skipped
    and no CSRF protection...
    """
    adapts(Interface, Interface)  # any context, any request

    # should be last lxml related transform
    order = 9000

    key_manager = None
    site = None
    safe_views = (
        'plone_lock_operations',
    )

    def __init__(self, published, request):
        self.published = published
        self.request = request

    def parseTree(self, result, encoding):
        # if it's a redirect, the result shall not be transformed
        request = self.request

        if request.response.status in (301, 302):
            return None

        if isinstance(result, XMLSerializer):
            return result

        # hhmmm, this is kind of taken right out of plone.app.theming
        # maybe this logic(parsing dom) should be someone central?
        contentType = self.request.response.getHeader('Content-Type')
        if contentType is None or not contentType.startswith('text/html'):
            return None

        contentEncoding = self.request.response.getHeader('Content-Encoding')
        if contentEncoding and contentEncoding in ('zip', 'deflate',
                                                   'compress',):
            return None

        if isinstance(result, list) and len(result) == 1:
            # do not parse empty strings to omit warning log message
            if not result[0].strip():
                return None
        try:
            result = getHTMLSerializer(
                result, pretty_print=False, encoding=encoding)
            # We are going to force html output here always as XHTML
            # output does odd character encodings
            result.serializer = html.tostring
            return result
        except (AttributeError, TypeError, etree.ParseError):
            # XXX handle something special?
            logger.warn('error parsing dom, failure to add csrf '
                        'token to response for url %s' % self.request.URL)
            return None

    def transformBytes(self, result, encoding):
        result = six.text_type(result, encoding, 'ignore')
        return self.transformIterable([result], encoding)

    def transformString(self, result, encoding):
        return self.transformIterable([result], encoding)

    def transformUnicode(self, result, encoding):
        return self.transformIterable([result], encoding)

    def transformIterable(self, result, encoding):
        """Apply the transform if required
        """
        # before anything, do the clickjacking protection
        if (
            X_FRAME_OPTIONS and
            not self.request.response.getHeader('X-Frame-Options')
        ):
            self.request.response.setHeader('X-Frame-Options', X_FRAME_OPTIONS)

        if CSRF_DISABLED:
            return

        # only auto CSRF protect authenticated users
        if isAnonymousUser(getSecurityManager().getUser()):
            return

        # if on confirm view, do not check, just abort and
        # immediately transform without csrf checking again
        if IConfirmView.providedBy(self.request.get('PUBLISHED')):
            # abort it, show the confirmation...
            transaction.abort()
            return self.transform(result, encoding)

        # next, check if we're a resource not connected
        # to a ZODB object--no context
        context = self.getContext()
        if not context:
            return

        try:
            tool = getToolByName(context, 'portal_url', None)
            if tool:
                self.site = tool.getPortalObject()
        except TypeError:
            self.site = getSite()

        try:
            self.key_manager = getUtility(IKeyManager)
        except ComponentLookupError:
            root = getRoot(context)
            self.key_manager = getRootKeyManager(root)

        if self.site is None and self.key_manager is None:
            # key manager not installed and no site object.
            # key manager must not be installed on site root, ignore
            return

        if not self.check():
            # we don't need to transform the doc, we're getting redirected
            return

        # finally, let's run the transform
        return self.transform(result, encoding)

    def getContext(self):
        published = self.request.get('PUBLISHED')
        if isinstance(published, types.MethodType):
            return published.im_self
        return aq_parent(published)

    def getViewName(self):
        try:
            return self.getContext().__name__
        except AttributeError:
            return None

    def check(self):
        """
        just being very careful here about our check so we don't
        cause errors that prevent this check from happening
        """
        try:
            return self._check()
        except:
            transaction.abort()
            logger.error("Error checking for CSRF. "
                         "Transaction will be aborted since the request "
                         "is now unsafe:\n%s" % (
                             traceback.format_exc()))
            raise

    def _registered_objects(self):
        app = self.request.PARENTS[-1]
        return list(itertools.chain.from_iterable([
            conn._registered_objects
            # skip the 'temporary' connection since it stores session objects
            # which get written all the time
            for name, conn in app._p_jar.connections.items()
            if name != 'temporary'
        ]))

    def _check(self):
        registered = self._registered_objects()
        if len(registered) > 0 and \
                not IDisableCSRFProtection.providedBy(self.request):
            if self.getViewName() in self.safe_views:
                return True
            # Okay, we're writing here, we need to protect!
            try:
                check(self.request, manager=self.key_manager)
                return True
            except ComponentLookupError:
                logger.info('Can not find key manager for CSRF protection. '
                            'This should not happen.')
                raise
            except Forbidden:
                # XXX
                # okay, so right now, we're going to check if the current
                # registered objects to write, are just portlet assignments.
                # I don't know why, but when a site is created, these
                # cause some writes on read. ALL, registered objects
                # need to be portlet assignments. XXX needs to be fixed
                # somehow...
                safe_oids = []
                if SAFE_WRITE_KEY in getattr(self.request, 'environ', {}):
                    safe_oids = self.request.environ[SAFE_WRITE_KEY]
                safe = True
                for obj in registered:
                    if IPortletAssignment.providedBy(obj):
                        continue
                    if getattr(obj, '_p_oid', False) in safe_oids:
                        continue
                    if ATBlob is not None and isinstance(obj, ATBlob):
                        continue
                    if isinstance(obj, OOBTree):
                        safe = False
                        for key in ANNOTATION_KEYS:
                            try:
                                if key in obj:
                                    safe = True
                                    break
                            except TypeError:
                                pass
                        if safe:
                            continue
                    safe = False
                    break
                if not safe:
                    if self.request.REQUEST_METHOD != 'GET':
                        # only try to be "smart" with GET requests
                        raise
                    logger.info(
                        '{0:s}\naborting transaction due to no CSRF '
                        'protection on url {1:s}'.format(
                            '\n'.join(traceback.format_stack()),
                            self.request.URL
                        )
                    )
                    transaction.abort()

                    # conditions for doing the confirm form are:
                    #   1. 301, 302 response code
                    #   2. text/html response
                    #   3. getSite could be none, zope root maybe, carry on
                    # otherwise,
                    #   just abort with a log entry because we tried to
                    #   write on read, without a POST request and we don't
                    #   know what to do with it gracefully.
                    resp = self.request.response
                    ct = resp.getHeader('Content-Type', '') or ''
                    if self.site and (
                            resp.status in (301, 302) or 'text/html' in ct):
                        data = self.request.form.copy()
                        data['original_url'] = self.request.URL
                        resp.redirect('%s/@@confirm-action?%s' % (
                            self.site.absolute_url(),
                            urlencode(data)
                        ))
                        return False
        return True

    def isActionInSite(self, action, current_url):
        # sanitize url
        url = urlparse(action)
        if not url.hostname:
            # no hostname means this is relative
            return True
        if url.hostname != current_url.hostname:
            return False
        return True

    def transform(self, result, encoding):

        result = self.parseTree(result, encoding)
        if result is None:
            return None
        root = result.tree.getroot()
        url = urlparse(self.request.URL)
        try:
            token = createToken(manager=self.key_manager)
        except ComponentLookupError:
            if self.site is not None:
                logger.warn(
                    'Keyring not found on site. This should not happen',
                    exc_info=True
                )
            return result

        for form in root.cssselect('form'):
            # XXX should we only do POST? If we're logged in and
            # it's an internal form, I'm inclined to say no...
            # method = form.attrib.get('method', 'GET').lower()
            # if method != 'post':
            #    continue

            # some get forms we definitely do not want to protect.
            # for now, we know search we do not want to protect
            method = form.attrib.get('method', 'GET').lower()
            action = form.attrib.get('action', '').strip()
            if method == 'get' and '@@search' in action:
                continue
            action = form.attrib.get('action', '').strip()
            if not self.isActionInSite(action, url):
                continue
            # check if the token is already on the form..
            hidden = form.cssselect('[name="_authenticator"]')
            if len(hidden) == 0:
                hidden = etree.Element("input")
                hidden.attrib['name'] = '_authenticator'
                hidden.attrib['type'] = 'hidden'
                hidden.attrib['value'] = token
                form.append(hidden)

        if self.site is not None and not root.cssselect('#protect-script'):
            # Alternative: add this in the resource registry.
            site_url = self.site.absolute_url()
            elements = root.cssselect('body')
            if len(elements):
                body = elements[0]
                protect_script = etree.Element("script")
                protect_script.attrib.update({
                    'type': "application/javascript",
                    'src': "%s/++resource++protect.js" % site_url,
                    'data-site-url': site_url,
                    'data-token': token,
                    'id': 'protect-script'
                })
                body.append(protect_script)

        return result



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from plone.protect import createToken
from plone.protect import CustomCheckAuthenticator
from plone.protect import protect
from plone.protect.authenticator import AuthenticatorView
from plone.protect.authenticator import check
from plone.protect.tests.case import KeyringTestCase
from plone.protect.tests.case import MockRequest
from unittest import makeSuite
from unittest import TestSuite
from zExceptions import Forbidden
from ZPublisher.HTTPRequest import HTTPRequest

import hmac
import sys


try:
    from hashlib import sha1 as sha
except ImportError:
    import sha


class AuthenticatorTests(KeyringTestCase):

    def setUp(self):
        KeyringTestCase.setUp(self)
        self.view = AuthenticatorView(None, None)

    def setUsername(self, name):
        user = getSecurityManager().getUser()
        user.name = name

    def setSecret(self, secret):
        self.manager['_forms'].data[0] = secret

    def testIsHtmlInput(self):
        auth = self.view.authenticator()
        self.failUnless(auth.startswith("<input"))
        self.failUnless(auth.endswith("/>"))

    def testDiffersPerUser(self):
        one = self.view.authenticator()
        self.setUsername("other")
        two = self.view.authenticator()
        self.assertNotEqual(one, two)

    def testDiffersPerSecret(self):
        one = self.view.authenticator()
        self.setSecret("other")
        two = self.view.authenticator()
        self.assertNotEqual(one, two)

    def testDiffersPerExtra(self):
        one = self.view.authenticator()
        two = self.view.authenticator('some-extra-value')
        self.assertNotEqual(one, two)


class VerifyTests(KeyringTestCase):

    key_size = 2

    def setUp(self):
        self.request = MockRequest()
        super(VerifyTests, self).setUp()
        self.view = AuthenticatorView(None, self.request)

    def setAuthenticator(self, key, extra='', name="_authenticator"):
        user = getSecurityManager().getUser().getUserName()
        auth = hmac.new(key, user + extra, sha).hexdigest()
        self.request[name] = auth

    def testCorrectAuthenticator(self):
        self.manager['_forms'].data[0] = "secret"
        self.setAuthenticator("secret")
        self.assertEqual(self.view.verify(), True)

    def testCustomAuthenticatorKeyName(self):
        self.manager['_forms'].data[0] = "secret"
        self.setAuthenticator("secret", name="_my_authenticator")
        self.assertEqual(self.view.verify(name="_my_authenticator"), True)

    def testOlderSecretVerifies(self):
        self.manager['_forms'].data[1] = "backup"
        self.setAuthenticator("backup")
        self.assertEqual(self.view.verify(), True)

    def testMissingAuthenticator(self):
        self.assertEqual(self.view.verify(), False)

    def testIncorrectAuthenticator(self):
        self.request["_authenticator"] = "incorrect"
        self.assertEqual(self.view.verify(), False)

    def testAuthenticatorWrongType(self):
        self.request["_authenticator"] = 123
        self.assertEqual(self.view.verify(), False)

    def testExtraArgumentCanBeVerified(self):
        self.manager['_forms'].data[0] = "secret"
        self.setAuthenticator("secret", 'some-extra-value')
        self.assertEqual(self.view.verify('some-extra-value'), True)


class DecoratorTests(KeyringTestCase):

    def setUp(self):
        self.request = HTTPRequest(sys.stdin, dict(SERVER_URL="dummy"), None)
        KeyringTestCase.setUp(self)

        def func(REQUEST=None):
            return 1
        self.func = protect(check)(func)

    def testNoRequestParameter(self):

        def func():
            pass
        self.assertRaises(ValueError, protect(check), func)

    def testIgnoreBadRequestType(self):
        self.assertEqual(self.func(), 1)

    def testBadAuthenticator(self):
        self.request["_authenticator"] = "incorrect"
        self.assertRaises(Forbidden, self.func, self.request)

    def testAuthenticatedCustom(self):
        self.request['_authenticator'] = createToken('some-value')

        def func(REQUEST=self.request):
            return True
        self.assertEquals(
            protect(CustomCheckAuthenticator('some-value'))(func)(), True)

    def testAuthenticatedCustomName(self):
        self.request['_my_authenticator'] = createToken('some-value')

        def func(REQUEST=self.request):
            return True
        self.assertEquals(
            protect(CustomCheckAuthenticator(
                'some-value', '_my_authenticator'))(func)(), True)


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(AuthenticatorTests))
    suite.addTest(makeSuite(VerifyTests))
    suite.addTest(makeSuite(DecoratorTests))
    return suite



# -*- coding: utf-8 -*-
from plone.protect.auto import safeWrite
from Products.PluggableAuthService import utils as pluggable_utils

import inspect


def wl_lockmapping(self, killinvalids=0, create=0):
    has_write_locks = hasattr(self, '_dav_writelocks')
    locks = self._old_wl_lockmapping(killinvalids=killinvalids, create=create)
    try:
        safeWrite(locks)
        if not has_write_locks and create:
            # first time writing to object, need to mark it safe
            safeWrite(self)
    except AttributeError:
        # not a persistent class, ignore
        pass
    return locks


def pluggableauth__getCSRFToken(request):
    """
    let plone.protect do it's job
    """
    return ''


def pluggableauth__checkCSRFToken(request, token='csrf_token', raises=True):
    """
    let plone.protect do it's job
    """
    pass


def marmoset_patch(func, replacement):
    source = inspect.getsource(replacement)
    exec source in func.func_globals
    func.func_code = replacement.func_code


# otherwise the patches do not get applied in some cases
if hasattr(pluggable_utils, 'checkCSRFToken'):
    marmoset_patch(pluggable_utils.checkCSRFToken,
                   pluggableauth__checkCSRFToken)
if hasattr(pluggable_utils, 'getCSRFToken'):
    marmoset_patch(pluggable_utils.getCSRFToken, pluggableauth__getCSRFToken)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '3.1.2'

setup(
    name='plone.protect',
    version=version,
    description="Security for browser forms",
    long_description='%s\n%s' % (
        open("README.rst").read(),
        open("CHANGES.rst").read()
    ),
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Framework :: Zope2",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='zope security CSRF',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.python.org/pypi/plone.protect',
    license='BSD',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['plone', ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'plone.keyring >= 3.0dev',
        'six',
        'zope.component',
        'zope.interface',
        'Zope2',
        'plone.transformchain',
        'repoze.xmliter>=0.3',
        'five.globalrequest',
        'collective.monkeypatcher'
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'Products.CMFPlone'
        ],
    }
)
